from ultralytics import YOLO
import torch
import cv2
from ultralytics.data.augment import LetterBox
from ultralytics.utils.plotting import Annotator, colors
from ultralytics.utils import ops
import numpy as np
import matplotlib.pyplot as plt
import ctypes.util
import gc
from ultralytics.utils.torch_utils import select_device
device = select_device("")

#FromFile
def imread(filename, flags=cv2.IMREAD_COLOR, dtype=np.uint8):
    try:
        n = np.fromfile(filename, dtype)
        img = cv2.imdecode(n, flags)
        return img
    except Exception as e:
        print(e)
        return None

class detectClass:
    def __init__(self,weights,imgsz):
        self.half = device.type != 'cpu'  # half precision only supported on CUDA
        # Load model
        self.model = YOLO(weights)
        self.imgsize = imgsz

    def __del___(self):
        self.model.to('cpu')
        self.model=None
        self.imgsize=None
        self.half=None
        self.iou_thres=None
        torch.cuda.empty_cache()

    def detectAI(self,img_ptrR,img_ptrG,img_ptrB, img_width, stride, img_height,thresh):
        np_imgR = np.squeeze(np.ctypeslib.as_array((stride * img_height * ctypes.c_uint8).from_address(img_ptrR)).reshape(1, img_height, stride),axis=0)
        np_imgG = np.squeeze(np.ctypeslib.as_array((stride * img_height * ctypes.c_uint8).from_address(img_ptrG)).reshape(1, img_height, stride),axis=0)
        np_imgB = np.squeeze(np.ctypeslib.as_array((stride * img_height * ctypes.c_uint8).from_address(img_ptrB)).reshape(1, img_height, stride),axis=0) # axis=0 is channel dimension 
        np_img = np.stack([np_imgB, np_imgG, np_imgR], 2)
        img = np.ascontiguousarray(np_img)
        preds = self.model.predict(img, save=False, imgsz=self.imgsize, conf=thresh)
        resultlist=[]
        det = getattr(preds[0], 'boxes', None)  # detections per imageNo0
        if len(det):
            # Write results
            clsL = getattr(det, 'cls', None).tolist()
            confL = getattr(det, 'conf', None).tolist()
            xyxyL = getattr(det, 'xyxy', None).tolist()
            for xyxy, conf, cls in zip(xyxyL,confL,clsL):
                #クラス番号/スコア/X始点/Y始点/X終点/Y終点
                resultlist.append([cls,conf,int(xyxy[0]),int(xyxy[1]),int(xyxy[2]),int(xyxy[3])])
        return resultlist
    
    def detectAIFile(self, path, img_width, stride, img_height,thresh):
        np_img = imread(path)  # BGR
        print(np_img.shape)
        # np_img = np_img[:, :, ::-1].transpose(2, 0, 1)  # BGR to RGB, to 3x416x416
        # print(np_img.shape)
        img = np.ascontiguousarray(np_img)
        preds = self.model.predict(img, save=False, imgsz=self.imgsize, conf=thresh)
        resultlist=[]
        det = getattr(preds[0], 'boxes', None)  # detections per imageNo0
        if len(det):
            # Write results
            clsL = getattr(det, 'cls', None).tolist()
            confL = getattr(det, 'conf', None).tolist()
            xyxyL = getattr(det, 'xyxy', None).tolist()
            for xyxy, conf, cls in zip(xyxyL,confL,clsL):
                #クラス番号/スコア/X始点/Y始点/X終点/Y終点
                resultlist.append([cls,conf,int(xyxy[0]),int(xyxy[1]),int(xyxy[2]),int(xyxy[3])])
                print(str(cls) +" "+ str(conf)+" " + str(int(xyxy[0]))+" "+ str(int(xyxy[1]))+" "+ str(int(xyxy[2]))+" "+ str(int(xyxy[3])))
        return resultlist


AIObject=[]

def open_YOLO(weights,imgsz,i):
    global AIObject
    try:
        if len(AIObject) > i:
            AIObject[i] = detectClass(weights,imgsz)
        else:
            AIObject.append(detectClass(weights,imgsz))
            i = len(AIObject) - 1
    except Exception as e:
        print(e)
        i = -1
    return i

def detect_YOLO(img_ptrR,img_ptrG,img_ptrB, img_width, stride, img_height,thresh,i):
    if len(AIObject) > i:
        resultlist = AIObject[i].detectAI(img_ptrR,img_ptrG,img_ptrB, img_width, stride, img_height,thresh)
    else:
        resultlist = -1
    return resultlist

def close_YOLO():
    global AIObject
    # del AIObject[:]
    AIObject.clear()
    gc.collect()
    torch.cuda.empty_cache()
    
def detect_YOLO_1_I(path, img_width, stride, img_height,thresh,i):
    if len(AIObject) > i:
        resultlist = AIObject[i].detectAIFile(path, img_width, stride, img_height,thresh)
    else:
        resultlist = -1
    return resultlist



if __name__ == '__main__':
    open_YOLO(r"E:\test_yolo\yolov8n-seg.pt",320,0)
    # print(1)
    try:
        rettime= detect_YOLO_1_I(r"E:\test_yolo\bus.jpg",810,810,1080,0.25,0)
    except Exception as e:
        print(e)
    close_YOLO()
    # Load a model
    # source = "E:/AIData/code_9/picture/pic_raw/NG/チョコ/学習用/221012_055439_0000000001_0_&Cam2Img.Normal.jpg"
    # model = YOLO('E:/AIData/code_9/AI_param/classification_weight/20231021_1124/try16/weights/best.pt')  # load an official model
    # point='E:/test'
    # Predict with the model
    # results = model.predict(source, save=False, imgsz=320, conf=0.5,project=point)
    # results = model.predict(source, save=False, imgsz=320, conf=0.5)

